@@ -7,16 +7,25 @@
 # Modified by Tony Byrne <fit4perl@byrnehq.com>
 
 package Test::C2FIT::eg::ArithmeticColumnFixture;
-
 use base 'Test::C2FIT::ColumnFixture';
+use Test::C2FIT::ScientificDouble;
+
 use strict;
 
+sub PI { 3.1415926535897932384626433832795; };
+
 sub new
 {
 	my $pkg = shift;
 	return $pkg->SUPER::new(x => 0, y => 0, @_);
 }
 
+sub suggestMethodResultType {
+    my ($self,$name) = @_;
+    return 'Test::C2FIT::ScientificDoubleTypeAdapter' if $name =~ /^(cos|sin)$/;
+    return $self->SUPER::suggestMethodResultType($name);
+}
+
 sub plus
 {
 	my $self = shift;
@@ -38,7 +47,7 @@ sub times
 sub divide
 {
 	my $self = shift;
-	return $self->{'x'} / $self->{'y'};
+	return int($self->{'x'} / $self->{'y'});
 }
 
 sub floating
@@ -47,4 +56,16 @@ sub floating
 	return $self->{'x'} / $self->{'y'};
 }
 
+sub sin
+{
+    my $x = $_[0]->{'x'};
+    return Test::C2FIT::ScientificDouble->new(sin( $x /180 * PI));
+}
+
+sub cos
+{
+    my $x = $_[0]->{'x'};
+    return Test::C2FIT::ScientificDouble->new(cos( $x /180 * PI));
+}
+
 1;
