@@ -1,4 +1,4 @@
-# $Id: PrimitiveFixture.pm,v 1.2 2005/04/27 13:16:29 tonyb Exp $
+# $Id: PrimitiveFixture.pm,v 1.3 2006/05/03 09:36:34 tonyb Exp $
 #
 # Copyright (c) 2002-2005 Cunningham & Cunningham, Inc.
 # Released under the terms of the GNU General Public License version 2 or later.
@@ -17,7 +17,7 @@ sub checkValue
 	my $self = shift;
 	my($cell, $value) = @_;
 
-	if ( Test::C2FIT::TypeAdapter::equals(undef, $cell->text(), $value) )
+	if ( Test::C2FIT::TypeAdapter->equals($cell->text(), $value) )
 	{
 		$self->right($cell);
 	} 
